import random
from . import sq1scrambler, trim

@trim
def get_squan():
    return sq1scrambler.call("sql_scrambler.getRandomScramble")

def get_clock():
  moves = ["0+ ","1+ ","2+ ","3+ ","4+ ","5+ ","6+ ","1- ","2- ","3- ","4- ","5- "]
  pins = ["UR","DR","DL","UL","U","R","D","L","ALL","U","R","D","L","ALL"]
  scramble = []

  for i in range(14):
      scramble.append(pins[i])
      scramble.append(random.choice(moves))

  def sort_l4m():
    l4m=["UR ","DR ","DL ","UL "]
    l4ms=[]
    for i in range(4):
      number=random.randint(1,2)
      if number==1:
        l4ms.append(l4m[i])
    return "".join(l4ms)

  last4=sort_l4m()
  scramble.append(last4)

  return "".join(scramble)


def get_skewb():
  y=random.randint(8,10)
  moves = ["B","R","L","U"]
  turns = [" ","' "]
  scramble = []
  usedmoves=[]
  usedmoves1=[]
  for i in range(y):

      if i % 2 ==0:

        randmoves=random.choice(moves)
        usedmoves.append(randmoves)
        scramble.append(randmoves)
        scramble.append(random.choice(turns))
        moves.remove(randmoves)
        if i != 0:
          moves.append(usedmoves1[-1])

      else:

        randmoves1=random.choice(moves)
        scramble.append(randmoves1)
        usedmoves1.append(randmoves1)
        scramble.append(random.choice(turns))
        moves.remove(randmoves1)
        moves.append(usedmoves[-1])
  return "".join(scramble)

def get_mega():
  turns=["-- ","++ "]
  umoves=["U'","U"]
  scramble=[]
  for i in range(7):
    scrambles=[]
    for x in range(5):
      for y in range(2):
        if y==0:
          scrambles.append("R")
        else:
          scrambles.append("D")
        scrambles.append(random.choice(turns))
    scrambles.append(random.choice(umoves))
    scramble.append("".join(scrambles))
  return "\n".join(scramble)


def get_pyra():
  y=random.randint(8,10)
  moves = ["B","R","L","U"]
  turns = [" ","' "]
  scramble = []
  usedmoves=[]
  usedmoves1=[]

  def sort_tips():

    tips=["l","r","b","u"]
    tip=[]

    for i in range(4):
      number=random.randint(1,2)
      if number==1:
        tip.append(tips[i])
        tip.append(random.choice(turns))

    return "".join(tip)


  for i in range(y):

      if i % 2 ==0:

        randmoves=random.choice(moves)
        usedmoves.append(randmoves)
        scramble.append(randmoves)
        scramble.append(random.choice(turns))
        moves.remove(randmoves)
        if i != 0:
          moves.append(usedmoves1[-1])

      else:

        randmoves1=random.choice(moves)
        scramble.append(randmoves1)
        usedmoves1.append(randmoves1)
        scramble.append(random.choice(turns))
        moves.remove(randmoves1)
        moves.append(usedmoves[-1])

  thetips=sort_tips()
  scramble.append(thetips)
  return "".join(scramble)



import execjs
from os import path

def trim(func):
    def trimmed_func(*args, **kwargs):
        return func(*args, **kwargs).strip()
    return trimmed_func


curr_dir = path.dirname(path.realpath(__file__))

with open(path.join(curr_dir, 'js/mathlib.js')) as f:
    mathlib = f.read()

with open(path.join(curr_dir, 'js/scramble.js')) as f:
    scramble = f.read()

with open(path.join(curr_dir, 'js/sq1.js')) as f:
    sq1 = f.read()

sq1scrambler   = execjs.compile(mathlib + scramble + sq1)



import random
class main():
  def get3():	
    moves = ["F","B","R","L","U","D"]	
    turns = [" ","2 ","' ","2 "]	
    scramble = []	
    usedmoves=[]	
    usedmoves1=[]	
    list1=["F","B"]	
    list2=["R","L"]	
    list3=["D","U"]	
    for i in range(20):	

      if i % 2 ==0:	
        randmoves=random.choice(moves)	
        if i>1:                   	
          while (randmoves in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves in list3 and scramble[-2] in list3 and scramble[-4] in list3):	
            randmoves=random.choice(moves)     	
        usedmoves.append(randmoves)	
        scramble.append(randmoves)	
        scramble.append(random.choice(turns))	
        moves.remove(randmoves)	
        if i != 0:	
          moves.append(usedmoves1[-1])	

      else:	

        randmoves1=random.choice(moves)	
        if i>1:	
          while (randmoves1 in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves1 in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves1 in list3 and scramble[-2] in list3 and scramble[-4] in list3):	
            randmoves1=random.choice(moves) 	

        scramble.append(randmoves1)	
        usedmoves1.append(randmoves1)	
        scramble.append(random.choice(turns))	
        moves.remove(randmoves1)	
        moves.append(usedmoves[-1])	
    return scramble
    
  def big(n=12,scramtype="big"):
    moves = ["F","B","R","L","U","D","Uw","Dw","Fw","Bw","Rw","Lw"]
    turns = [" ","2 ","' "]
    moveslist = ["F","B","R","L","U","D"]
    scramble = []
    usedmoves=[]
    usedmoves1=[]
    ruf=["R","U","F"]
    list1=["U","D","Uw","Dw"]
    list2=["R","L","Lw","Rw"]
    list3=["F","B","Fw","Bw"]
    try:
      n1=int(n/2)
    except TypeError:
      raise TypeError("Invalid number")
    if n% 2==0:
      times=int(n/2)
      for ru in ruf:

        moves.append(f"{n1}{ru}w")
        if ru=="R":
          list2.append(f"{n1}{ru}w")
        elif ru=="U":
          list1.append(f"{n1}{ru}w")
        elif ru=="F":
          list3.append(f"{n1}{ru}w")

    else:
        times=int(n/2)+1
    for i in range(3,times):
      for move in moveslist:
          moves.append(f"{i}{move}w")
          if move=="U" or move=="D":
              list1.append(f"{i}{move}w")
          elif move=="R" or move=="L":
              list2.append(f"{i}{move}w")
          elif move=="B" or move=="F":
              list3.append(f"{i}{move}w")
    if n<8:
      num=n*20-40
    else:
      num=n*15
    for i in range(num):

      if i % 2 ==0:
        randmoves=random.choice(moves)
        if i>1:
          if (randmoves in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves in list3 and scramble[-2] in list3 and scramble[-4] in list3):
            while randmoves==scramble[-4]:
              randmoves=random.choice(moves)
          if i>2:
            while (randmoves in list1 and scramble[-2] in list1 and scramble[-4] in list1 and scramble[-6] in list1) or (randmoves in list2 and scramble[-2] in list2 and scramble[-4] in list2 and scramble[-6] in list2) or (randmoves in list3 and scramble[-2] in list3 and scramble[-4] in list3 and scramble[-6] in list3):
              randmoves=random.choice(moves)
        usedmoves.append(randmoves)
        scramble.append(randmoves)
        scramble.append(random.choice(turns))
        moves.remove(randmoves)
        if i != 0:
          moves.append(usedmoves1[-1])

      else:
        randmoves1=random.choice(moves)
        if i>2:
          randmoves1=random.choice(moves)
          if (randmoves1 in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves1 in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves1 in list3 and scramble[-2] in list3 and scramble[-4] in list3):
            while randmoves1==scramble[-4]:
              randmoves1=random.choice(moves) 
          while (randmoves1 in list1 and scramble[-2] in list1 and scramble[-4] in list1 and scramble[-6] in list1) or (randmoves1 in list2 and scramble[-2] in list2 and scramble[-4] in list2 and scramble[-6] in list2) or (randmoves1 in list3 and scramble[-2] in list3 and scramble[-4] in list3 and scramble[-6] in list3):
            randmoves1=random.choice(moves)         


        scramble.append(randmoves1)
        usedmoves1.append(randmoves1)
        scramble.append(random.choice(turns))
        moves.remove(randmoves1)
        moves.append(usedmoves[-1])
    if scramtype=="big":
      return "".join(scramble)
    elif scramtype=="bld":
      biggest=int(n/2)+1
      possi=[]
      possi2=[]
      if biggest % 2==0:
        possi=["","x ","x' ","x2 ","z ","z2 ","z' "]
        possi2=["y","y'","y2",""]
      else:
        for move in ruf:
          if move !="U":
            possi.append(f"{biggest}{move}w ")
            possi.append(f"{biggest}{move}w' ")
            possi.append(f"{biggest}{move}w2 ")
          else:
            possi2.append(f"{biggest}{move}w ")
            possi2.append(f"{biggest}{move}w' ")
            possi2.append(f"{biggest}{move}w2 ")
      scramble.append(random.choice(possi))
      scramble.append(random.choice(possi2))
      return "".join(scramble)
    else:
      raise ValueError("Invalid scramble type")
def get1():
  moves = ["x","y","z"]
  turns = [" ","2 ","' "]
  scramble = []
  usedmoves=[]
  usedmoves1=[]
  for i in range(8):
                
      if i % 2 ==0:
                        
        randmoves=random.choice(moves)
        usedmoves.append(randmoves)
        scramble.append(randmoves)
        scramble.append(random.choice(turns))
        moves.remove(randmoves)
        if i != 0:
          moves.append(usedmoves1[-1])

      else:
                    
        randmoves1=random.choice(moves)
        scramble.append(randmoves1)
        usedmoves1.append(randmoves1)
        scramble.append(random.choice(turns))
        moves.remove(randmoves1)
        moves.append(usedmoves[-1])
  return "".join(scramble)




def get2(): 
  moves = ["F","R","U"]
  turns = [" ","2 ","' "]
  scramble = []
  usedmoves=[]
  usedmoves1=[]
  num=random.randint(8,12)
  for i in range(num):
                
      if i % 2 ==0:
                        
        randmoves=random.choice(moves)
        usedmoves.append(randmoves)
        scramble.append(randmoves)
        scramble.append(random.choice(turns))
        moves.remove(randmoves)
        if i != 0:
          moves.append(usedmoves1[-1])

      else:
                    
        randmoves1=random.choice(moves)
        scramble.append(randmoves1)
        usedmoves1.append(randmoves1)
        scramble.append(random.choice(turns))
        moves.remove(randmoves1)
        moves.append(usedmoves[-1])
  return "".join(scramble)

def get3(scramtype="3x3"):
  a=main.get3()
  scramtype=scramtype.lower()
  if scramtype=="3x3":
    return "".join(a)
  elif scramtype=="bld":
    scramble=[]
    scramble3=a
    for item in scramble3:
      scramble.append(item)
    possi=["","Rw ","Rw' ","Rw2 ","Fw ","Fw2 ","Fw' "]
    possi2=["Uw","Uw'","Uw2",""]
    scramble.append(random.choice(possi))
    scramble.append(random.choice(possi2))
    return "".join(scramble)
  elif scramtype=="fmc":
    a="".join(a).split()
    list1=["F","F2","F'","B'","B2","B"]
    list2=["R2","R","R'","L","L'","L2"]
    list3=["F","F2","F'"]
    list4=["R2","R","R'"]
    times=0
    while (a[0] in list1 and a[1] in list1) or (a[0] in list3) or (a[-1] in list2 and a[-2] in list2) or (a[-1] in list4):
      if times>3:
        a=main.get3()
        a="".join(a).split()
        times=0

      if a[0] in list3:
        a.remove(a[0])
        times+=1

      if (a[0] in list1 and a[1] in list1):
        if a[0] not in list3:
          a.remove(a[1])
          times+=1

      
      if a[-1] in list4:
        a.remove(a[-1])
        times+=1
      
      if (a[-1] in list2 and a[-2] in list2):
        if a[-1] not in list4:
          a.remove(a[-2])
          times+=1

    scramble="R' U' F " + " ".join(a) + " R' U' F"
    return scramble
  else:
    raise ValueError("Invalid scramble type")


def get4(scramtype="4x4"):
  moves=["F","B","R","L","U","D"]
  wide_turns=["F","Fw","B","R","Rw","L","D","U","Uw"]
  turns = [" ","2 ","' ","2 "]
  scramble = []
  usedmoves=[]
  usedmoves1=[]
  list1=["F","B","Fw"]
  list2=["R","L","Rw"]
  list3=["D","U","Uw"]
  scramble=main.get3()
  for i in range(25):
                    
    if i % 2 ==0:
      randmoves=random.choice(wide_turns)
      if randmoves in moves and scramble[-2] in moves and scramble[-4] in moves:
        randmoves=random.choice(wide_turns)
      if (randmoves in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves in list3 and scramble[-2] in list3 and scramble[-4] in list3):
        while randmoves==scramble[-4]:
          randmoves=random.choice(moves)
      while (randmoves in list1 and scramble[-2] in list1 and scramble[-4] in list1 and scramble[-6] in list1) or (randmoves in list2 and scramble[-2] in list2 and scramble[-4] in list2 and scramble[-6] in list2) or (randmoves in list3 and scramble[-2] in list3 and scramble[-4] in list3 and scramble[-6] in list3):
        randmoves=random.choice(wide_turns)
      usedmoves.append(randmoves)
      scramble.append(randmoves)
      scramble.append(random.choice(turns))
      wide_turns.remove(randmoves)
      if i != 0:
        wide_turns.append(usedmoves1[-1])
    else:                  
      randmoves1=random.choice(wide_turns)
      if randmoves1 in moves and scramble[-2] in moves and scramble[-4] in moves:
        randmoves1=random.choice(wide_turns)
      if (randmoves1 in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves1 in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves1 in list3 and scramble[-2] in list3 and scramble[-4] in list3):
        while randmoves1==scramble[-4]:
          randmoves1=random.choice(moves)
      while (randmoves1 in list1 and scramble[-2] in list1 and scramble[-4] in list1) or (randmoves1 in list2 and scramble[-2] in list2 and scramble[-4] in list2) or (randmoves1 in list3 and scramble[-2] in list3 and scramble[-4] in list3):
        randmoves1=random.choice(wide_turns)
  
      scramble.append(randmoves1)
      usedmoves1.append(randmoves1)
      scramble.append(random.choice(turns))
      wide_turns.remove(randmoves1)
      wide_turns.append(usedmoves[-1])
  if scramtype=="4x4":
    return "".join(scramble)
  elif scramtype=="bld":
    possi=["","x ","x' ","x2 ","z ","z2 ","z' "]
    possi2=["y","y'","y2",""]
    scramble.append(random.choice(possi))
    scramble.append(random.choice(possi2))
    return "".join(scramble)
  else:
    raise ValueError("Invalid scramble type")

def get5(scramtype="5x5"):

  if scramtype=="5x5":
    scramble=main.big(5)
  elif scramtype=="bld":
    scramble=main.big(5,"bld")
  else:
    raise ValueError("Invalid scramble type")
  return scramble


def get6():
  scramble=main.big(6)
  return scramble

def get7():
  scramble=main.big(7)
  return scramble

def get8():
  scramble=main.big(8)
  return scramble

def get9():
  scramble=main.big(9)
  return scramble

def get10():
  scramble=main.big(10)
  return scramble

def get11():
  scramble=main.big(11)
  return scramble

def get12():
  scramble=main.big(12)
  return scramble

def get_big_cube(n=12,scramtype="big"):
  scramble=main.big(n,scramtype)
  return scramble



import random

def get_ivy():
    moves = ["L", "R", 'D', 'B']
    turns= [' ', "' "]
    scramble = []
    usedmoves = []
    usedmoves1 = []
    
    for i in range(8):

        if i % 2 == 0:
            
            randmoves = random.choice(moves)
            usedmoves.append(randmoves)
            scramble.append(randmoves)
            scramble.append(random.choice(turns))
            moves.remove(randmoves)

            if i != 0:
                moves.append(usedmoves1[-1])

        else:
            randmoves1 = random.choice(moves)
            scramble.append(randmoves1)
            usedmoves1.append(randmoves1)
            scramble.append(random.choice(turns))
            moves.remove(randmoves1)
            moves.append(usedmoves[-1])
    return "".join(scramble)


def get223():
    num_moves = random.randint(8, 10)
    double_moves = ["R", "F"]
    single_moves = ["U", "D"]
    turns = ["2 ", " ", "' "]
    scramble = []

    for i in range(num_moves):

        if i % 2 == 0:
            randmoves = random.choice(double_moves)
            scramble.append(randmoves)
            turn ="2 "

            scramble.append(turn)
        else:
            randmoves1 = random.choice(single_moves)
            scramble.append(randmoves1)
            scramble.append(random.choice(turns))

    return "".join(scramble)


def get_kilo():
  turns=["-- ","++ "]
  umoves=["U'","U"]
  scramble=[]
  for i in range(4):
    scrambles=[]
    for x in range(5):
      for y in range(2):
        if y==0:
          scrambles.append("R")
        else:
          scrambles.append("D")
        scrambles.append(random.choice(turns))
    scrambles.append(random.choice(umoves))
    if i!=3:
      scrambles.append(" x2 ")
    scramble.append("".join(scrambles))
  return "\n".join(scramble)


def get_redi():
    turns = [" ", "' "]
    scramble = []

    for i in range(8):
        scrambles = []
        num_moves = random.randint(2, 5)
        for x in range(num_moves):
            for y in range(2):
                if y == 0:
                    scrambles.append("R")
                else:
                    scrambles.append("L")
                scrambles.append(random.choice(turns))
        if i != 7:
            scrambles.append("x")
        scramble.append("".join(scrambles))

    return "\n".join(scramble)

def get113():
    moves = ["B", "F", "R", "L"]
    turns = ["' ", " "]
    usedmoves = []
    usedmoves1 = []
    num_moves = random.randint(3, 6)
    scramble = []

    for i in range(num_moves):
        if i % 2 == 0:
            randmoves = random.choice(moves)
            scramble.append(randmoves)
            scramble.append(random.choice(turns))
            usedmoves.append(randmoves)
            moves.remove(randmoves)

            if i != 0:
                moves.append(usedmoves1[-1])

        else:
            randmoves1 = random.choice(moves)
            scramble.append(randmoves1)
            scramble.append(random.choice(turns))
            usedmoves1.append(randmoves1)
            moves.remove(randmoves1)
            moves.append(usedmoves[-1])

    return "".join(scramble)

def get_masterpyra():
    num_moves = random.randint(38, 42)
    moves = ["R", "L", "U", "B", "Uw", "Lw", "Bw", "Rw"]
    turns = [" ", "' "]
    list1 = ["U", "Uw"]
    list2 = ["L", "Lw"]
    list3 = ["R", "Rw"]
    list4 = ["B", "Bw"]
    scramble = []
    usedmoves = []
    usedmoves1 = []

    def sort_tips():

        tips = ['u', 'b', 'r', 'l']
        tip = []

        for x in range(4):
            number = random.randint(1, 2)
            if number == 1:
                tip.append(tips[x])
                tip.append(random.choice(turns))

        return "".join(tip)

    for i in range(num_moves):

        if i % 2 == 0:
            randmoves = random.choice(moves)
            if i > 1:

                while (randmoves in list1 and scramble[-2] in list1) or (randmoves in list2 and scramble[-2] in list2) \
                        or (randmoves in list3 and scramble[-2] in list3) or (randmoves in list4 and scramble[-2] in
                                                                              list4):
                    randmoves = random.choice(moves)

            usedmoves.append(randmoves)
            scramble.append(randmoves)
            scramble.append(random.choice(turns))
            moves.remove(randmoves)
            if i != 0:
                moves.append(usedmoves1[-1])

        else:
            randmoves1 = random.choice(moves)
            if i > 1:

                while (randmoves1 in list1 and scramble[-2] in list1) or (randmoves1 in list2 and scramble[-2] in list2)\
                        or (randmoves1 in list3 and scramble[-2] in list3) or (randmoves1 in list4 and scramble[-2] in list4):
                    randmoves1 = random.choice(moves)
            scramble.append(randmoves1)
            usedmoves1.append(randmoves1)
            scramble.append(random.choice(turns))
            moves.remove(randmoves1)
            moves.append(usedmoves[-1])

    thetips = sort_tips()
    scramble.append(thetips)
    return "".join(scramble)

def get233():
    scramble = []
    usedmoves = []
    usedmoves1 = []
    moves = ["R2 ", "L2 ", "F2 ", "B2 ", "U"]
    turns = [" ", "' ", "2 "]

    for i in range(25):
        if i % 2 == 0:

            randmove = random.choice(moves)
            scramble.append(randmove)
            usedmoves.append(randmove)

            if randmove == "U":
                scramble.append(random.choice(turns))

            moves.remove(randmove)

            if i != 0:
                moves.append(usedmoves1[-1])

        else:
            randmove1 = random.choice(moves)
            scramble.append(randmove1)
            usedmoves1.append(randmove1)
            moves.remove(randmove1)
            moves.append(usedmoves[-1])

            if randmove1 == "U":
                scramble.append(random.choice(turns))

    return "".join(scramble)



import setuptools

setuptools.setup(
    name="pycubescrambler",
    version="0.2.6",
    author="midnightcuberx",
    author_email="midnightcuberx@gmail.com",
    url="https://github.com/midnightcuberx/pycubescrambler",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'appdirs',
        'packaging',
        'PyExecJS',
        'pyparsing',
        'six',
    ],
)
